@@ -14,6 +14,7 @@
 #include <stdio.h>
 #include <math.h>
 #include <stdlib.h>
+#include <errno.h>
 
 #ifdef SUNOS
 #include <strings.h>
@@ -43,51 +44,85 @@ extern void i_hsv_to_rgbf(i_fcolor *color);
 extern void i_rgb_to_hsv(i_color *color);
 extern void i_hsv_to_rgb(i_color *color);
 
-i_img *im_img_8_new(pIMCTX, i_img_dim x,i_img_dim y,int ch);
-#define i_img_empty(im, x, y) i_img_empty_ch((im), (x), (y), 3)
-i_img *im_img_empty_ch(pIMCTX, i_img *im,i_img_dim x,i_img_dim y,int ch);
-#define i_img_empty_ch(im, x, y, ch) im_img_empty_ch(aIMCTX, (im), (x), (y), (ch))
+i_img *im_img_8_new_extra(pIMCTX, i_img_dim x,i_img_dim y,int ch, int extrach);
 void   i_img_exorcise(i_img *im);
 void   i_img_destroy(i_img *im);
 i_img *im_img_alloc(pIMCTX);
+i_img *im_img_new(pIMCTX, const i_img_vtable *vtbl, i_img_dim xsize, i_img_dim ysize,
+                  int channels, int extra, i_img_bits_t bits);
 void im_img_init(pIMCTX, i_img *im);
+void i_img_refcnt_inc(i_img *im);
+i_image_data_alloc_t *
+i_img_data_fallback(i_img *im, i_data_layout_t layout, i_img_bits_t bits, unsigned flags,
+                    void **pdata, size_t *psize, int *extra);
+i_image_data_alloc_t *
+i_new_image_data_alloc_def(i_img *im);
+i_image_data_alloc_t *
+i_new_image_data_alloc_free(i_img *im, void *releaseme);
+
+/* fallback handlers for i_[gp]samp_(bits)? */
+extern i_img_dim
+i_gsampf_fp(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_fsample_t *samp, 
+            int const *chans, int chan_count);
+extern i_img_dim
+i_psampf_fp(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, const i_fsample_t *samp, 
+            int const *chans, int chan_count);
+extern i_img_dim
+i_gsamp_bits_fb(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y, unsigned *samp,
+                const int *chans, int chan_count, int bits);
+extern i_img_dim
+i_psamp_bits_fb(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y, const unsigned *samp,
+                const int *chans, int chan_count, int bits);
+extern i_img_dim
+i_gslin_fallback(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y,
+                 i_sample16_t *samp, const int *chans, int chan_count);
+extern i_img_dim
+i_gslinf_fallback(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y,
+                  i_fsample_t *samp, const int *chans, int chan_count);
+extern i_img_dim
+i_pslin_fallback(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y,
+                 const i_sample16_t *samps, const int *chans, int chan_count);
+extern i_img_dim
+i_pslinf_fallback(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y,
+                  const i_fsample_t *samps, const int *chans, int chan_count);
+
+/* wrapper functions that forward palette calls to the underlying image,
+   assuming the underlying image is the first pointer in whatever
+   ext_data points at
+*/
+extern int i_addcolors_forward(i_img *im, const i_color *, int count);
+extern int i_getcolors_forward(i_img *im, int i, i_color *, int count);
+extern int i_colorcount_forward(i_img *im);
+extern int i_maxcolors_forward(i_img *im);
+extern int i_findcolor_forward(i_img *im, const i_color *color, 
+			       i_palidx *entry);
+extern int i_setcolors_forward(i_img *im, int index, const i_color *colors, 
+                               int count);
 
 void   i_img_info(i_img *im,i_img_dim *info);
 
 extern i_img *i_sametype(i_img *im, i_img_dim xsize, i_img_dim ysize);
-extern i_img *i_sametype_chans(i_img *im, i_img_dim xsize, i_img_dim ysize, int channels);
+extern i_img *i_sametype_chans_extra(i_img *im, i_img_dim xsize, i_img_dim ysize, int channels, int extra);
+
+extern int i_img_check_entries(i_img *im);
 
 /* Image feature settings */
 
 void   i_img_setmask    (i_img *im,int ch_mask);
 int    i_img_getmask    (i_img *im);
-int    i_img_getchannels(i_img *im);
-i_img_dim i_img_get_width(i_img *im);
-i_img_dim i_img_get_height(i_img *im);
-i_color_model_t i_img_color_model(i_img *im);
-int i_img_alpha_channel(i_img *im, int *channel);
-int i_img_color_channels(i_img *im);
+
+extern const imcms_curve_t *im_model_curves(pIMCTX, i_color_model_t model, int *color_chans);
 
 /* Base functions */
 
-extern int (i_ppix)(i_img *im,i_img_dim x,i_img_dim y, const i_color *val);
-extern int (i_gpix)(i_img *im,i_img_dim x,i_img_dim y,i_color *val);
-extern int (i_ppixf)(i_img *im,i_img_dim x,i_img_dim y, const i_fcolor *val);
-extern int (i_gpixf)(i_img *im,i_img_dim x,i_img_dim y,i_fcolor *val);
-
-extern i_img_dim (i_plin)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
+extern i_img_dim i_plin(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
 			  const i_color *vals);
-extern i_img_dim (i_glin)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
+extern i_img_dim i_glin(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
 			  i_color *vals);
-extern i_img_dim (i_plinf)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
+extern i_img_dim i_plinf(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
 			   const i_fcolor *vals);
-extern i_img_dim (i_glinf)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, 
+extern i_img_dim i_glinf(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, 
 			   i_fcolor *vals);
-extern i_img_dim (i_gsamp)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
-			   i_sample_t *samp, const int *chans, int chan_count);
-extern i_img_dim
-(i_gsampf)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_fsample_t *samp,
-	   const int *chans, int chan_count);
 extern i_img_dim
 (i_gpal)(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y, i_palidx *vals);
 extern i_img_dim
@@ -149,7 +184,8 @@ i_compose(i_img *out, i_img *src,
 	       i_img_dim width, i_img_dim height, int combine, double opacity);
 
 extern i_img *
-i_combine(i_img **src, const int *channels, int in_count);
+im_combine(pIMCTX, i_img **src, const int *channels, int in_count);
+#define i_combine(src, channels, count) im_combine(aIMCTX, (src), (channels), (count))
 
 undef_int i_flipxy (i_img *im, int direction);
 extern i_img *i_rotate90(i_img *im, int degrees);
@@ -158,6 +194,8 @@ extern i_img *i_rotate_exact_bg(i_img *im, double amount, const i_color *backp,
 extern i_img *i_matrix_transform(i_img *im, i_img_dim xsize, i_img_dim ysize, const double *matrix);
 extern i_img *i_matrix_transform_bg(i_img *im, i_img_dim xsize, i_img_dim ysize, const double *matrix,  const i_color *backp, const i_fcolor *fbackp);
 
+extern int i_trim_rect(i_img *im, double transp_threshold, int color_count, const i_trim_colors_t *colors,
+		       i_img_dim *left, i_img_dim *top, i_img_dim *right, i_img_dim *bottom);
 void i_bezier_multi(i_img *im,int l,const double *x,const double *y,const i_color *val);
 int i_poly_aa     (i_img *im,int l,const double *x,const double *y,const i_color *val);
 int i_poly_aa_cfill(i_img *im,int l,const double *x,const double *y,i_fill_t *fill);
@@ -178,12 +216,13 @@ undef_int i_flood_cfill_border(i_img *im, i_img_dim seedx, i_img_dim seedy, i_fi
 
 /* image processing functions */
 
-int i_gaussian    (i_img *im, double stdev);
+int i_gaussian    (i_img *im, double stddev);
+int i_gaussian2    (i_img *im, double stddevX, double stddevY);
 int i_conv        (i_img *im,const double *coeff,int len);
 void i_unsharp_mask(i_img *im, double stddev, double scale);
 
 /* colour manipulation */
-extern i_img *i_convert(i_img *src, const double *coeff, int outchan, int inchan);
+extern i_img *i_convert(i_img *src, const double *coeff, int outchan, int extraoutchans, int inchan, int linear);
 extern void i_map(i_img *im, unsigned char (*maps)[256], unsigned int mask);
 
 float i_img_diff   (i_img *im1,i_img *im2);
@@ -220,17 +259,33 @@ extern i_img *i_img_to_pal(i_img *src, i_quantize *quant);
 extern i_img *i_img_to_rgb(i_img *src);
 extern i_img *i_img_masked_new(i_img *targ, i_img *mask, i_img_dim x, i_img_dim y, 
                                i_img_dim w, i_img_dim h);
-extern i_img *im_img_16_new(pIMCTX, i_img_dim x, i_img_dim y, int ch);
+extern i_img *im_img_16_new_extra(pIMCTX, i_img_dim x, i_img_dim y, int ch, int extra);
 extern i_img *i_img_to_rgb16(i_img *im);
-extern i_img *im_img_double_new(pIMCTX, i_img_dim x, i_img_dim y, int ch);
+extern i_img *im_img_double_new_extra(pIMCTX, i_img_dim x, i_img_dim y, int ch, int extra);
 extern i_img *i_img_to_drgb(i_img *im);
+extern i_img *im_img_float_new_extra(pIMCTX, i_img_dim x, i_img_dim y, int ch, int extra);
+extern i_img *i_img_to_float_rgb(i_img *im);
+
+extern i_img *
+im_lin_img_16_new_extra(pIMCTX, i_img_dim x, i_img_dim y, int ch, int extra);
+extern i_img *i_img_to_linrgb16(i_img *im);
+extern i_img *
+im_lin_img_double_new_extra(pIMCTX, i_img_dim x, i_img_dim y, int ch, int extra);
+extern i_img *i_img_to_linrgbdbl(i_img *im);
 
 extern int i_img_is_monochrome(i_img *im, int *zero_is_white);
 extern int i_get_file_background(i_img *im, i_color *bg);
 extern int i_get_file_backgroundf(i_img *im, i_fcolor *bg);
 
-const char * i_test_format_probe(io_glue *data, int length);
+const char * im_test_format_probe(im_context_t ctx, io_glue *data, int length);
+#define i_test_format_probe(io, length) im_test_format_probe(aIMCTX, (io), (length))
 
+/* file type magic to extend file detection */
+extern int im_add_file_magic(im_context_t ctx, const char *name,
+			     const unsigned char *bits, const unsigned char *mask,
+			     size_t length);
+#define i_add_file_magic(name, bits, mask, length) \
+  im_add_file_magic(aIMCTX, (name), (bits), (mask), (length))
 
 i_img   * i_readraw_wiol(io_glue *ig, i_img_dim x, i_img_dim y, int datachannels, int storechannels, int intrl);
 undef_int i_writeraw_wiol(i_img* im, io_glue *ig);
@@ -285,6 +340,7 @@ void i_turbnoise(i_img *im,double xo,double yo,double scale);
 void i_gradgen(i_img *im, int num, i_img_dim *xo, i_img_dim *yo, i_color *ival, int dmeasure);
 int i_nearest_color(i_img *im, int num, i_img_dim *xo, i_img_dim *yo, i_color *ival, int dmeasure);
 i_img *i_diff_image(i_img *im, i_img *im2, double mindist);
+i_img *i_rgbdiff_image(i_img *im, i_img *im2);
 int
 i_fountain(i_img *im, double xa, double ya, double xb, double yb, 
            i_fountain_type type, i_fountain_repeat repeat, 
@@ -345,15 +401,18 @@ im_int_check_image_file_limits(im_context_t ctx, i_img_dim width, i_img_dim heig
 
 /* memory allocation */
 void* mymalloc(size_t size);
+void *myzmalloc(size_t size);
 void  myfree(void *p);
 void* myrealloc(void *p, size_t newsize);
 void* mymalloc_file_line (size_t size, char* file, int line);
+void* myzmalloc_file_line (size_t size, const char* file, int line);
 void  myfree_file_line   (void *p, char*file, int line);
 void* myrealloc_file_line(void *p, size_t newsize, char* file,int line);
 
 #ifdef IMAGER_DEBUG_MALLOC
 
 #define mymalloc(x) (mymalloc_file_line((x), __FILE__, __LINE__))
+#define myzmalloc(x) (myzmalloc_file_line((x), __FILE__, __LINE__))
 #define myrealloc(x,y) (myrealloc_file_line((x),(y), __FILE__, __LINE__))
 #define myfree(x) (myfree_file_line((x), __FILE__, __LINE__))
 
@@ -409,4 +468,8 @@ extern void i_mutex_unlock(i_mutex_t m);
 
 #include "imio.h"
 
+#include "imcms.h"
+
+#include "iminline.h"
+
 #endif
